@@ -65,8 +65,15 @@ struct pinentry pinentry =
     0,		/* Enhanced mode.  */
     1,		/* Global grab.  */
     0,		/* Parent Window ID.  */
+    NULL,       /* Touch file.  */
     0,		/* Result.  */
-    0           /* Locale error flag. */
+    0,          /* Locale error flag. */
+    0,          /* One-button flag.  */
+    PINENTRY_COLOR_DEFAULT,
+    0,
+    PINENTRY_COLOR_DEFAULT,
+    PINENTRY_COLOR_DEFAULT,
+    0
   };
 
 
@@ -255,7 +262,10 @@ pinentry_init (const char *pgmname)
 int
 pinentry_have_display (int argc, char **argv)
 {
-  if (getenv ("DISPLAY"))
+  const char *s;
+
+  s = getenv ("DISPLAY");
+  if (s && *s)
     return 1;
   for (; argc; argc--, argv++)
     if (!strcmp (*argv, "--display"))
@@ -285,6 +295,56 @@ usage (void)
 }
 
 
+char *
+parse_color (char *arg, pinentry_color_t *color_p, int *bright_p)
+{
+  static struct
+  {
+    const char *name;
+    pinentry_color_t color;
+  } colors[] = { { "none", PINENTRY_COLOR_NONE },
+		 { "default", PINENTRY_COLOR_DEFAULT },
+		 { "black", PINENTRY_COLOR_BLACK },
+		 { "red", PINENTRY_COLOR_RED },
+		 { "green", PINENTRY_COLOR_GREEN },
+		 { "yellow", PINENTRY_COLOR_YELLOW },
+		 { "blue", PINENTRY_COLOR_BLUE },
+		 { "magenta", PINENTRY_COLOR_MAGENTA },
+		 { "cyan", PINENTRY_COLOR_CYAN },
+		 { "white", PINENTRY_COLOR_WHITE } };
+
+  int i;
+  char *new_arg;
+  pinentry_color_t color = PINENTRY_COLOR_DEFAULT;
+
+  if (!arg)
+    return NULL;
+
+  new_arg = strchr (arg, ',');
+  if (new_arg)
+    new_arg++;
+
+  if (bright_p)
+    {
+      const char *bname[] = { "bright-", "bright", "bold-", "bold" };
+
+      *bright_p = 0;
+      for (i = 0; i < sizeof (bname) / sizeof (bname[0]); i++)
+	if (!strncasecmp (arg, bname[i], strlen (bname[i])))
+	  {
+	    *bright_p = 1;
+	    arg += strlen (bname[i]);
+	  }
+    }
+
+  for (i = 0; i < sizeof (colors) / sizeof (colors[0]); i++)
+    if (!strncasecmp (arg, colors[i].name, strlen (colors[i].name)))
+      color = colors[i].color;
+
+  *color_p = color;
+  return new_arg;
+}
+
 /* Parse the command line options.  Returns 1 if user should print
    version and exit.  Can exit the program if only help output is
    requested.  */
@@ -304,6 +364,7 @@ pinentry_parse_opts (int argc, char *argv[])
      { "enhanced", no_argument,          0, 'e' },
      { "no-global-grab", no_argument,    0, 'g' },
      { "parent-wid", required_argument,  0, 'W' },
+     { "colors", required_argument,	 0, 'c' },
      { "help", no_argument,              0, 'h' },
      { "version", no_argument, &opt_version, 1 },
      { NULL, 0, NULL, 0 }};
@@ -375,6 +436,14 @@ pinentry_parse_opts (int argc, char *argv[])
 	  /* FIXME: Add some error handling.  Use strtol.  */
 	  break;
 
+	case 'c':
+	  optarg = parse_color (optarg, &pinentry.color_fg,
+				&pinentry.color_fg_bright);
+	  optarg = parse_color (optarg, &pinentry.color_bg, NULL);
+	  optarg = parse_color (optarg, &pinentry.color_so,
+				&pinentry.color_so_bright);
+	  break;
+
         default:
           fprintf (stderr, "%s: oops: option not handled\n", this_pgmname);
 	  break;
@@ -450,6 +519,14 @@ option_handler (ASSUAN_CONTEXT ctx, const char *key, const char *value)
       pinentry.parent_wid = atoi (value);
       /* FIXME: Use strtol and add some error handling.  */
     }
+  else if (!strcmp (key, "touch-file"))
+    {
+      if (pinentry.touch_file)
+        free (pinentry.touch_file);
+      pinentry.touch_file = strdup (value);
+      if (!pinentry.touch_file)
+	return ASSUAN_Out_Of_Core;
+    }
   else
     return ASSUAN_Invalid_Option;
   return 0;
@@ -576,6 +653,7 @@ cmd_getpin (ASSUAN_CONTEXT ctx, char *line)
       set_prompt = 1;
     }
   pinentry.locale_err = 0;
+  pinentry.one_button = 0;
 
   result = (*pinentry_cmd_handler) (&pinentry);
   if (pinentry.error)
@@ -613,11 +691,19 @@ cmd_getpin (ASSUAN_CONTEXT ctx, char *line)
 }
 
 
+/* Note that the option --one-button is hack to allow the use of old
+   pinentries while the caller is ignoring the result.  Given that
+   options have never been used or flagged as an error the new option
+   is an easy way to enable the messsage mode while not requiring to
+   update pinentry or to have the caller test for the message
+   command.  New applications which are free to require an updated
+   pinentry should use MESSAGE instead. */
 static int
 cmd_confirm (ASSUAN_CONTEXT ctx, char *line)
 {
   int result;
 
+  pinentry.one_button = !!strstr (line, "--one-button");
   pinentry.locale_err = 0;
   result = (*pinentry_cmd_handler) (&pinentry);
   if (pinentry.error)
@@ -628,7 +714,29 @@ cmd_confirm (ASSUAN_CONTEXT ctx, char *line)
 
   return result ? 0
                 : (pinentry.locale_err? ASSUAN_Locale_Problem
-                                      : ASSUAN_Not_Confirmed);
+                                      : (pinentry.one_button 
+                                         ? 0
+                                         : ASSUAN_Not_Confirmed));
+}
+
+
+static int
+cmd_message (ASSUAN_CONTEXT ctx, char *line)
+{
+  int result;
+
+  pinentry.one_button = 1;
+  pinentry.locale_err = 0;
+  result = (*pinentry_cmd_handler) (&pinentry);
+  if (pinentry.error)
+    {
+      free (pinentry.error);
+      pinentry.error = NULL;
+    }
+
+  return result ? 0 
+                : (pinentry.locale_err? ASSUAN_Locale_Problem
+                                      : 0);
 }
 
 
@@ -650,6 +758,7 @@ register_commands (ASSUAN_CONTEXT ctx)
       { "SETCANCEL",  0,  cmd_setcancel },
       { "GETPIN",     0,  cmd_getpin },
       { "CONFIRM",    0,  cmd_confirm },
+      { "MESSAGE",    0,  cmd_message },
       { NULL }
     };
   int i, j, rc;
